"use strict";
// SPDX-FileCopyrightText: 2025 Gnuxie <Gnuxie@protonmail.com>
//
// SPDX-License-Identifier: AFL-3.0
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserMayJoinRoomEndpoint = exports.UserMayJoinRoomRequestBody = void 0;
const typebox_1 = require("@sinclair/typebox");
const matrix_protection_suite_1 = require("matrix-protection-suite");
const log = new matrix_protection_suite_1.Logger("UserMayJoinRoomEndpoint");
exports.UserMayJoinRoomRequestBody = typebox_1.Type.Object({
    user: matrix_protection_suite_1.StringUserIDSchema,
    room: matrix_protection_suite_1.StringRoomIDSchema,
    is_invited: typebox_1.Type.Boolean(),
});
class UserMayJoinRoomEndpoint {
    constructor(pluginManager) {
        this.pluginManager = pluginManager;
        // nothing to do.
    }
    async handleUserMayJoinRoomAsync(request, response, isResponded) {
        const decodedBody = matrix_protection_suite_1.Value.Decode(exports.UserMayJoinRoomRequestBody, request.body);
        if ((0, matrix_protection_suite_1.isError)(decodedBody)) {
            log.error("Error decoding request body:", decodedBody.error);
            if (!isResponded && this.pluginManager.isBlocking()) {
                response.status(400).send({
                    errcode: "M_INVALID_PARAM",
                    error: "Error handling user, room, and is_invited",
                });
            }
            return;
        }
        if (!isResponded && this.pluginManager.isBlocking()) {
            const blockingResult = await this.pluginManager.callBlockingHandles(decodedBody.ok);
            if (blockingResult === "NOT_SPAM") {
                response.status(200);
                response.send({});
            }
            else {
                response.status(400);
                response.send(blockingResult);
            }
        }
        else if (!isResponded) {
            response.status(200);
            response.send({});
        }
        this.pluginManager.callNonBlockingHandlesInTask(decodedBody.ok);
    }
    handleUserMayJoinRoom(request, response) {
        if (!this.pluginManager.isBlocking()) {
            response.status(200);
            response.send({});
        }
        void (0, matrix_protection_suite_1.Task)(this.handleUserMayJoinRoomAsync(request, response, !this.pluginManager.isBlocking()));
    }
}
exports.UserMayJoinRoomEndpoint = UserMayJoinRoomEndpoint;
//# sourceMappingURL=UserMayJoinRoomEndpoint.js.map